require 'ostruct'
require File.expand_path("../git/git_ssh", __FILE__)

module Evrone
  module CI
    module SCM

      class Git

        include Common::Helper::Shell

        COMMIT_RE = /^(.*) -:- (.*) \((.*)\) -:- (.*)$/

        attr_reader :src, :sha, :path, :logger, :git_ssh

        def initialize(src, sha, path, options = {}, &block)
          @src      = src
          @sha      = sha
          @path     = path
          @git_ssh  = GitSSH.new options[:deploy_key]
          @logger   = block
        end

        def fetch
          code = git_ssh.open do
            repo_exist? ? update : clone
          end
          code = checkout if code == 0
          code
        end

        def self.make_export_command(from, to)
          %{ (cd '#{from}' && git checkout-index -a -f --prefix='#{to}/') }.strip
        end

        def make_fetch_command
          %{ (test -d #{path} || git clone -q #{src} #{path}) && cd #{path} && git checkout -qf #{sha} }.strip
        end

        def commit_info
          rs = {}
          if str = commit_info_string
            if m = str.match(COMMIT_RE)
              rs.merge!(
                sha:     m[1],
                author:  m[2],
                email:   m[3],
                message: m[4]
              )
            end
          end
          OpenStruct.new rs
        end

        private

          def commit_info_string
            output = ""
            code = spawn commit_info_cmd, chdir: path do |io|
              output << io
            end
            if code == 0
              output.strip
            else
              nil
            end
          end

          def commit_info_cmd
            %{git log -1 --pretty=format:'%H -:- %cn (%ce) -:- %s'}
          end

          def repo_exist?
            File.directory?(path.to_s + "/.git")
          end

          def update
            run_git 'git fetch origin', chdir: path
          end

          def clone
            run_git "git clone -q #{src} #{path}"
          end

          def checkout
            run_git "git checkout -qf #{sha}", chdir: path
          end

          def run_git(cmd, options = {})
            env = {
              'GIT_SSH' => git_ssh.location.path
            }
            logger.call "$ #{cmd}\n"
            spawn(env, cmd, options, &logger)
          end

      end

    end
  end
end
